C
      LOGICAL FUNCTION CKCOLR(MP, NP, DP, P, LDP1, LDP2, ZERCOL, Q, LDQ,
     *                        W, TOL, IERR)
C
C     PURPOSE
C
C     To check whether the polynomial matrix
C                                                    dp-1            dp
C        P(s) = P(0) + P(1) * s + . . . + P(dp-1) * s     + P(dp) * s  ,
C
C     is column reduced.
C
C     ARGUMENTS IN
C
C        MP - INTEGER.
C            The number of rows of the polynomial matrix P(s).
C            MP >= 1.
C        NP - INTEGER.
C            The number of columns of the polynomial matrix P(s).
C            NP >= 1.
C        DP - INTEGER.
C            The degree of the polynomial matrix P(s).
C            DP >= 0.
C        P - DOUBLE PRECISION array of DIMENSION (LDP1,LDP2,DP+1).
C            The leading MP by NP by (DP+1) part of this array must
C            contain the coefficients of the polynomial matrix P(s).
C            Specifically, P(i,j,k) must contain the coefficient of
C            s**(k-1) of the polynomial which is the (i,j)-th element
C            of P(s), where i = 1,2,...,MP, j = 1,2,...,NP and
C            k = 1,2,...,DP+1.
C        LDP1 - INTEGER.
C            The leading dimension of array P as declared in the calling
C            program.
C            LDP1 >= MP.
C        LDP2 - INTEGER.
C            The second dimension of array P as declared in the calling
C            program.
C            LDP2 >= NP.
C
C        ARGUMENTS OUT
C
C        ZERCOL - LOGICAL array of DIMENSION at least (NP).
C            If ZERCOL(j) = .TRUE. then the j-th column of P(s) is zero;
C            otherwise the j-th column belongs to P1(s) (see METHOD).
C
C        WORKSPACE
C
C        Q - DOUBLE PRECISION array of DIMENSION (LDQ,NP).
C        LDQ - INTEGER.
C            The leading dimension of array Q as declared by the calling
C            program.
C            LDQ >= MP.
C        W - DOUBLE PRECISION array of DIMENSION (MP).
C
C     TOLERANCES
C
C        TOL - DOUBLE PRECISION.
C            A tolerance below which matrix elements are considered to
C            be zero. If the user sets TOL to be less than
C            EPS * (((DP+1)*MP)**2 * MAX(P(i,j,k))), then the tolerance
C            is taken as EPS * (((DP+1)*MP)**2 * MAX(P(i,j,k))),
C            i = 1,...,MP, j = 1,...,NP, k = 1,...,DP+1, where EPS is
C            the machine precision.
C
C     ERROR INDICATOR
C
C        IERR - INTEGER.
C           Unless the routine detects an error (see next section),
C           IERR contains 0 on exit.
C
C     WARNINGS AND ERRORS DETECTED BY THE ROUTINE
C
C        IERR = 1 : Invalid input parameter(s).
C
C     METHOD
C
C     Let GAMC(P) be the contant matrix such that each of its columns
C     contains the coefficients of the highest power of s occurring
C     in the corresponding column of P(s), the so-called leading column
C     coefficient matrix. Then P(s) is called column reduced if there
C     exists a permutation matrix T such that P(s) = ( Z , P1(s) ) * T,
C     where Z is a zero matrix and GAMC(P1) has full column rank.
C
C     The algorithm used, which is in fact the QR decomposition of the
C     leading column coefficient matrix, is as follows:
C     The columns of the leading column coefficient matrix of P1(s) are
C     determined one by one, where a column is considered zero if its
C     Euclidean norm is less than TOL. To each new column the
C     Householder transformations are applied that have transformed the
C     submatrix of the former columns in upper triangular form. If the
C     new column is independent of its predecessors, then a new
C     Householder transformation is generated and applied such that the
C     augmented matrix is upper triangular.
C     The routine terminates after the last column of P(s) has been
C     treated or when the new column is dependent of its predecessors.
C
C     CONTRIBUTOR
C
C        A.J. Geurts (Eindhoven University of Technology).
C        C. Praagman (University of Groningen).
C
C     REVISIONS
C
C        1994, February 11.
C        1996, May 28.
C
      IMPLICIT NONE
C     .. Parameters ..
      DOUBLE PRECISION ZERO
      PARAMETER (ZERO = 0.0D0)
C     .. Scalar Arguments ..
      INTEGER MP, NP, DP, LDP1, LDP2, LDQ, IERR
      DOUBLE PRECISION TOL
C     .. Array Arguments ..
      DOUBLE PRECISION P(LDP1,LDP2,*), Q(LDQ,*), W(*)
      LOGICAL ZERCOL(*)
C     .. Local Scalars ..
      INTEGER H, J, J1, K
      DOUBLE PRECISION EPS, TOLER, NORM, TAU
      LOGICAL FULLRK, NOTCJ
C     .. External Subroutines/Functions ..
      EXTERNAL DCOPY, DLAMCH, DLANGE, DLARFG, DLARFV, DLASET, DNRM2
      DOUBLE PRECISION DLAMCH, DLANGE, DNRM2
C     .. Intrinsic Functions ..
      INTRINSIC DBLE, MAX, MIN
C
C     .. Executable Statements ..
C
C     Check input parameters
C
      IF (MP.LT.1 .OR. NP.LT.1 .OR. DP.LT.0 .OR. LDP1.LT.MP .OR.
     *    LDP2.LT.NP) THEN
          IERR = 1
          RETURN
      END IF
C
C     Computation of the tolerance. EPS is the machine precision.
C
      EPS = DLAMCH('E')
      TOLER = ZERO
      DO 10 K = 1, DP + 1
         TOLER = MAX(TOLER, DLANGE('M', MP, NP, P(1,1,K), LDP1, W))
   10 CONTINUE

      TOLER = DBLE(((DP+1) * MP))**2 * TOLER * EPS
      IF (TOLER .LT. TOL) TOLER = TOL
C
      CALL DLASET('G', MP, NP, ZERO, ZERO, Q, LDQ)
      J = 1
      J1 = 1
      FULLRK = .TRUE.
C     WHILE (FULLRK and J1 <= NP) DO
   20 IF (FULLRK .AND. J1.LE.NP) THEN
C
C        Find the j-th column of the leading column coefficient matrix
C        of P1(s) and put it in W.
C
         K = DP + 1
         NOTCJ = .TRUE.
C        WHILE (j-th column not found) DO
   30    IF (NOTCJ .AND. K.GE.1) THEN
            NORM = DNRM2(MP, P(1,J1,K), 1)
            IF (NORM .GE. TOLER) THEN
               CALL DCOPY(MP, P(1,J1,K), 1, W, 1)
               NOTCJ = .FALSE.
            END IF
            K = K - 1
            GO TO 30
         END IF
C        END WHILE 30
C
C        Check whether the j-th column is linearly independent of the
C        preceeding columns.
C
         IF (NOTCJ) THEN
            ZERCOL(J1) = .TRUE.
            J1 = J1 + 1
         ELSE
            ZERCOL(J1) = .FALSE.
C
C           Apply the Householder transformations Qh,
C           h = 1,...,min(mp,j) - 1, to W.
C
            DO 40 H = 1, MIN(MP,J) - 1
               CALL DLARFV(MP-H+1, Q(H+1,H), 1, Q(H,H), W(H), W(H+1), 1)
   40       CONTINUE
            NORM = DNRM2(MP-J+1, W(J), 1)
            IF (NORM .LT. TOLER) THEN
               FULLRK = .FALSE.
            ELSE
C
C              Generate the Householder transformation Qj.
C
               IF (J .LT. MP) THEN
                  CALL DLARFG(MP-J+1, W(J), W(J+1), 1, TAU)
                  CALL DCOPY(MP-J, W(J+1), 1, Q(J+1,J), 1)
                  Q(J,J) = TAU
               ELSE
                  Q(J,J) = ZERO
               END IF
            END IF
            J = J + 1
            J1 = J1 + 1
         END IF
         GO TO 20
      END IF
C     END WHILE 20
      CKCOLR = FULLRK
      RETURN
C *** Last line of CKCOLR ***
      END

