C
      LOGICAL FUNCTION CKGAMC(MP, INV, GAMC, LDG, Q, LDQ, RWORK, TOL)
C
C     PURPOSE
C
C     To check whether the leading coefficient matrix has still full
C     column rank after a new column has been appended.
C     REMARK: This auxiliary routine is intended to be called only from
C             the routine COLRED.
C
C     ARGUMENTS IN
C
C        MP - INTEGER.
C            The number of rows of the matrix GAMC.
C            MP >= 1.
C        INV - INTEGER.
C            The number of columns of the matrix GAMC.
C            INV >= 1.
C        GAMC - DOUBLE PRECISION array of DIMENSION (LDG,INV)
C            The leading MP by INV part of this array must contain the
C            leading coefficient matrix GAMC of which the first INV - 1
C            columns have been transformed in upper triangular form.
C            Note: this array is overwritten
C        LDG - INTEGER.
C            The leading dimension of array GAMC as declared in the
C            calling program.
C            LDG >= MP.
C        Q - DOUBLE PRECISION array of DIMENSION (LDQ,INV)
C            The leading MP by INV - 1 part of this array must contain
C            the constants and the vectors of the elementary Householder
C            transformations, as supplied by DLARFG, by which the first
C            INV - 1 columns of GAMC have been transformed into an upper
C            triangular matrix.
C        LDQ - INTEGER.
C            The leading dimension of array Q as declared in the calling
C            program.
C            LDQ >= MP.
C
C     ARGUMENTS OUT
C
C        GAMC - DOUBLE PRECISION array of DIMENSION (LDG,INV)
C            The leading MP by INV part of this array contains the
C            leading coefficient matrix GAMC transformed in upper
C            triangular form.
C        Q - DOUBLE PRECISION array of DIMENSION (LDQ,INV)
C            The leading MP by INV part of this array contains the
C            constants and the vectors of the elementary Householder
C            transformations by which GAMC has been transformed into an
C            upper triangular matrix.
C
C        WORKSPACE
C
C        RWORK - DOUBLE PRECISION array of DIMENSION (MP)
C
C     TOLERANCES
C
C        TOL - DOUBLE PRECISION.
C            A tolerance below which matrix elements are considered to
C            be zero.
C
C     METHOD
C
C     Let the first INV - 1 columns of GAMC be linearly independent,
C     which has been checked by former calls of CKGAMC. A new column is
C     appended. To this column the Householder transformations are
C     applied that have transformed the matrix of the former columns in
C     upper triangular form. If the new column is independent of its
C     predecessors, then a new Householder transformation is generated
C     and applied such that the augmented matrix is upper triangular.
C
C     CONTRIBUTOR
C
C        A.J. Geurts.
C
C     REVISIONS
C
C        1993, October 29.
C        1994, December 8.
C
      IMPLICIT NONE
C     .. Parameters ..
      DOUBLE PRECISION ZERO
      PARAMETER (ZERO = 0.0D0)
C     .. Scalar Arguments ..
      INTEGER MP, INV, LDG, LDQ
      DOUBLE PRECISION TOL
C     .. Array Arguments ..
      DOUBLE PRECISION GAMC(LDG,*), Q(LDQ,*), RWORK(*)
C     .. Local Scalars ..
      INTEGER J
      DOUBLE PRECISION NORM, TAU
C     .. External Subroutines/Functions ..
      EXTERNAL DCOPY, DLARFG, DLARFV, DLAVEC, DNRM2
      DOUBLE PRECISION DNRM2
C
C     .. Executable Statements ..
C
      IF (INV .LE. MP) THEN
         CALL DLAVEC(MP, ZERO, Q(1,INV), 1)
C
C        Check whether the INV-th column is linearly independent of the
C        preceeding columns by applying the Householder transformations
C        Q(h), h = 1,...,INV-1, to the INV-th column.
C
         CALL DCOPY(MP, GAMC(1,INV), 1, RWORK, 1)
         DO 10 J = 1, INV - 1
            CALL DLARFV(MP-J+1, Q(J+1,J), 1, Q(J,J), RWORK(J),
     *                  RWORK(J+1), 1)
   10    CONTINUE
         NORM = DNRM2(MP-INV+1, RWORK(INV), 1)
         IF (NORM .LT. TOL) THEN
            CKGAMC = .FALSE.
         ELSE
C
C           Generate the Householder transformation Q(INV) if INV < MP.
C
            CALL DCOPY(MP, RWORK, 1, GAMC(1,INV), 1)
            IF (INV .LT. MP) THEN
               CALL DLARFG(MP-INV+1, RWORK(INV), RWORK(INV+1), 1, TAU)
               GAMC(INV,INV) = RWORK(INV)
               CALL DLAVEC(MP-INV, ZERO, GAMC(INV+1,INV), 1)
               CALL DCOPY(MP-INV, RWORK(INV+1), 1, Q(INV+1,INV), 1)
               Q(INV,INV) = TAU
            ELSE
               Q(INV,INV) = ZERO
            END IF
            CKGAMC = .TRUE.
         END IF
      ELSE
         CKGAMC = .FALSE.
      END IF
      RETURN
C *** Last line of CKGAMC ***
      END

