C
      SUBROUTINE COLRED(MP, NP, DP, P, LDP1, LDP2, DR, DU, R, LDR1,
     *                  LDR2, U, LDU1, LDU2, ZERCOL, IWORK, RWORK,
     *                  TOL, IERR)
C
C     PURPOSE
C
C     To compute for a given polynomial matrix
C                                                     dp-1            dp
C        P(s) = P(0) + P(1) * s + . . . + P(dp-1) * s     + P(dp) * s  ,
C
C     a unimodular polynomial matrix U(s) such that R(s) = P(s) * U(s)
C     is column reduced.
C
C     ARGUMENTS IN
C
C        MP - INTEGER.
C            The number of rows of the polynomial matrix P(s).
C            MP >= 1.
C        NP - INTEGER.
C            The number of columns of the polynomial matrix P(s).
C            NP >= 1.
C        DP - INTEGER.
C            The degree of the polynomial matrix P(s).
C            DP >= 0.
C        P - DOUBLE PRECISION array of DIMENSION (LDP1,LDP2,DP+1).
C            The leading MP by NP by (DP+1) part of this array must
C            contain the coefficients of the polynomial matrix P(s).
C            Specifically, P(i,j,k) must contain the coefficient of
C            s**(k-1) of the polynomial which is the (i,j)-th element
C            of P(s), where i = 1,2,...,MP, j = 1,2,...,NP and
C            k = 1,2,...,DP+1.
C        LDP1 - INTEGER.
C            The leading dimension of array P as declared in the calling
C            program.
C            LDP1 >= MP.
C        LDP2 - INTEGER.
C            The second dimension of array P as declared in the calling
C            program.
C            LDP2 >= NP.
C
C     ARGUMENTS OUT
C
C        DR - INTEGER.
C            The degree of the column reduced polynomial matrix R(s).
C        DU - INTEGER.
C            The degree of the unimodular polynomial matrix U(s).
C        R - DOUBLE PRECISION array of DIMENSION (LDR1,LDR2,DP+1).
C            The leading MP by NP by (DR+1) part of this array contains
C            the coefficients of the column reduced polynomial matrix
C            R(s). Specifically, R(i,j,k) contains the coefficient of
C            s**(k-1) of the polynomial which is the (i,j)-th element of
C            R(s), where i = 1,2,...,MP, j = 1,2,...,NP and
C            k = 1,2,...,DR+1.
C        LDR1 - INTEGER.
C            The leading dimension of array R as declared in the calling
C            program.
C            LDR1 >= MP.
C        LDR2 - INTEGER.
C            The second dimension of array R as declared in the calling
C            program.
C            LDR2 >= NP.
C        U - DOUBLE PRECISION array of DIMENSION (LDU1,LDU2,NP*DP+1).
C            The leading NP by NP by (DU+1) part of this array contains
C            the coefficients of the unimodular polynomial matrix U(s).
C            Specifically, U(i,j,k) contains the coefficient of s**(k-1)
C            of the polynomial which is the (i,j)-th element of U(s),
C            where i = 1,2,...,NP, j = 1,2,...,NP and k = 1,2,...,DU+1.
C        LDU1 - INTEGER.
C            The leading dimension of array U as declared in the calling
C            program.
C            LDU1 >= NP.
C        LDU2 - INTEGER.
C            The second dimension of array U as declared in the calling
C            program.
C            LDU2 >= NP.
C        ZERCOL - LOGICAL array of DIMENSION at least (NP).
C            If ZERCOL(j) = .TRUE. then the j-th column of R(s) is zero;
C            otherwise the j-th column belongs to R1(s) (see METHOD).
C
C     WORKSPACE
C
C        IWORK - INTEGER array of DIMENSION at least (liwork),
C            where liwork = 2*mamax + 2 * NP + 1,
C            and mamax = (NP*DP+1) * MP.
C        RWORK - DOUBLE PRECISION array of DIMENSION at least (lrwork),
C            where lrwork = (2*mamax+NP*DP+5) * namax + mamax + (2*MP+1) * NP,
C            and namax = mamax + NP.
C                           .
C     TOLERANCES
C
C        TOL - DOUBLE PRECISION.
C            A tolerance below which matrix elements are considered to
C            be zero. If the user sets TOL to be less than
C            EPS * (((DP+1)*MP)**2 * MAX(P(i,j,k))), then the tolerance
C            is taken as EPS * (((DP+1)*MP)**2 * MAX(P(i,j,k))),
C            i = 1,...,MP, j = 1,...,NP, k = 1,..., DP + 1, where EPS is
C            the machine precision.
C
C     ERROR INDICATOR
C
C        IERR - INTEGER.
C           Unless the routine detects an error (see next section),
C           IERR contains 0 on exit.
C
C     WARNINGS AND ERRORS DETECTED BY THE ROUTINE
C
C        IERR = 1 : On entry, MP < 1 or NP < 1 or DP < 0 or
C                   LDP1 < MP or LDP2 < NP or
C                   LDR1 < MP or LDR2 < NP or
C                   LDU1 < NP or LDU2 < NP.
C        IERR = 2 : P(s) is not column reduced, and no column reduced 
C                   R(s) has been found.
C
C     METHOD
C
C     Let GAMC(P) be the constant matrix such that each of its columns
C     contains the coefficients of the highest power of s occurring in
C     the corresponding column of P(s), the so-called leading column
C     coefficient matrix. Then P(s) is called column reduced if there
C     exists a permutation matrix T such that P(s) = ( Z , P1(s) ) * T,
C     where Z is a zero matrix and GAMC(P1) has full column rank.
C
C     Let (U(s),Z(s))' be a minimal polynomial basis (MPB) for
C          b
C     Ker(s P(s), -I), for some b > 0. It has been proved, see [1], that
C     if b is greater than d'c(P), the sum of all but the smallest
C                                                                -b
C     column degrees of P(s), then U(s) is unimodular and R(s) = s  Z(s)
C     is column reduced and P(s) * U(s) = R(s).
C                                           b
C     The routine uses a linearization of (s P(s), -I) to compute an MPB
C     for b = 1,2,... and checks for each b whether R(s) is column
C     reduced, i.e. whether GAMC(R1) has full column rank. The algorithm
C     finishes with U(s) and R(s) as soon as R(s) is column reduced.
C
C     REFERENCES
C
C     [1] Neven, W.H.L. and Praagman, C.
C         Column Reduction of Polynomial Matrices.
C         Linear Algebra and its Applications 188, 189, pp. 569-589,
C         1993.
C     [2] Geurts, A.J. and Praagman, C.
C         A Fortran subroutine for column reduction of polynomial
C         matrices. EUT Report 94-WSK-01, Eindhoven, June 1994.
C
C     NUMERICAL ASPECTS
C
C     The algorithm used by the routine involves the construction of a
C                                                    b
C     special staircase form of a linearization of (s  P(s), -I) with
C     pivots considered to be non-zero when they are greater than or
C     equal to TOL. These pivots are then inverted in order to construct
C                         b
C     the columns of ker(s  P(s), -I).
C     The user is recommended to choose TOL of the order of the relative
C     error in the elements of P(s). If TOL is chosen to be too small,
C     then a very small element of insignificant value may be taken as
C     pivot. As a consequence, the correct null-vectors, and hence R(s),
C     may not be found. In the case that R(s) has not been found and in
C     the case that the elements of the computed U(s) and R(s) are large
C     relative to the elements of P(s) the user should consider trying
C     several values of TOL.
C
C     CONTRIBUTORS
C
C        A.J. Geurts (Eindhoven University of Technology).
C        C. Praagman (University of Groningen).
C
C     REVISIONS
C
C        1994, February 11.
C        1996, June 5.
C
      IMPLICIT NONE
C     .. Parameters ..
      DOUBLE PRECISION ZERO, ONE
      PARAMETER (ZERO = 0.0D0, ONE = 1.0D0)
C     .. Scalar Arguments ..
      INTEGER MP, NP, DP, LDP1, LDP2, DR, DU, LDR1, LDR2, LDU1, LDU2,
     *        IERR
      DOUBLE PRECISION TOL
C     .. Array Arguments ..
      INTEGER IWORK(*)
      DOUBLE PRECISION P(LDP1,LDP2,*), R(LDR1,LDR2,*), U(LDU1,LDU2,*),
     *                 RWORK(*)
      LOGICAL ZERCOL(*)
C     .. Local Scalars ..
      INTEGER LDA, LDAB, LDQ, LDY, LDG, MU, S, SK, A, AB, Q, Y, YI,
     *        GAMC, BMAX, DP1, K, MAMAX, NAMAX
      DOUBLE PRECISION EPS, TOLER
      LOGICAL COLRDC, PKZERO
C     .. External Subroutines/Functions ..
      EXTERNAL COLRD1, CKCOLR, DLACPY, DLAMCH, DLANGE, DLASET
      DOUBLE PRECISION DLAMCH, DLANGE
      LOGICAL CKCOLR
C     .. Intrinsic Functions ..
      INTRINSIC DBLE, MAX
C
C     .. Executable Statements ..
C
C     Check the input parameters.
C
      IF (MP.LT.1 .OR. NP.LT.1 .OR. DP.LT.0 .OR. LDP1.LT.MP .OR.
     *   LDP2.LT.NP .OR. LDR1.LT.MP .OR. LDR2.LT.NP .OR.
     *   LDU1.LT.NP .OR. LDU2.LT.NP) THEN
         IERR = 1
         RETURN
      END IF
C
      IERR = 0
C
C     Computation of the tolerance. EPS is the machine precision.
C
      EPS = DLAMCH('E')
      TOLER = ZERO
      DO 10 K = 1, DP + 1
         TOLER = MAX(TOLER, DLANGE('M', MP, NP, P(1,1,K), LDP1, RWORK))
   10 CONTINUE
C
      TOLER = DBLE(((DP+1) * MP)**2) * TOLER * EPS
      IF (TOLER .LT. TOL) TOLER = TOL
C
C     Computation of the true degree of P(s).
C
      K = DP + 2
      PKZERO = .TRUE.
C     WHILE (P(k) is a zero matrix) DO
   20 IF (PKZERO .AND. (K.GT.1)) THEN
         K = K - 1
         PKZERO = (DLANGE('M', MP, NP, P(1,1,K), LDP1, RWORK) .EQ. ZERO)
         GO TO 20
      END IF
C     END WHILE 20
      DP1 = K - 1
C
C     Check whether P(s) is already column reduced.
C
      Q = MP + 1
      LDQ = MP
      COLRDC = CKCOLR(MP, NP, DP1, P, LDP1, LDP2, ZERCOL, RWORK(Q), LDQ,
     *                RWORK, TOLER, IERR)
      IF (COLRDC) THEN
         DR = DP1
         DO 30 K = 1, DR + 1
            CALL DLACPY('G', MP, NP, P(1,1,K), LDP1, R(1,1,K), LDR1)
   30    CONTINUE
         DU = 0
         CALL DLASET('G', NP, NP, ZERO, ONE, U, LDU1)
         RETURN
      END IF
C
      BMAX = (NP - 1) * DP1 + 1
      MAMAX = (DP1 + BMAX) * MP
      NAMAX = MAMAX + NP
      LDA = MAMAX + 1
      LDAB = LDA
      LDY = NAMAX
      LDG = MP
      MU = 1
      S = MU + MAMAX + 1
      SK = S + MAMAX
      A = 1 + MAMAX + NAMAX 
      AB = A + (MAMAX + 1) * NAMAX
      Q = AB + (MAMAX + 1) * NAMAX
      Y = Q + MP * NP
      YI = Y + (BMAX + DP1 + 1) * NAMAX
      GAMC = YI + NP
C
      CALL COLRD1(MP, NP, DP1, P, LDP1, LDP2, DR, DU, R, LDR1, LDR2,
     *            U, LDU1, LDU2, ZERCOL, IWORK(MU), IWORK(S), IWORK(SK),
     *            RWORK(A), LDA, RWORK(AB), LDAB, RWORK(Q), LDQ,
     *            RWORK(Y), LDY, RWORK(YI), RWORK(GAMC), LDG, RWORK,
     *            TOLER, IERR)
C
C     Check whether the computed R(s) is column reduced.
C
      IF (IERR .EQ. 0) THEN
         Q = MP + 1
         COLRDC = CKCOLR(MP, NP, DR, R, LDR1, LDR2, ZERCOL, RWORK(Q),
     *                   LDQ, RWORK, TOL, IERR)
         IF (.NOT. COLRDC) IERR = 2
      ELSE
         IERR = 2
      END IF
      RETURN
C *** Last line of COLRED ***
      END
