C
      SUBROUTINE COMPTV(MA, NA, I, J, A, LDA, S, Y, LDY, RWORK)
C
C     PURPOSE
C
C     To apply the Householder reflections, thus far applied to sA - E,
C     to the null vector Y(s) corresponding to a given column of the
C     transformed sA' - E, which transforms this null vector into a null
C     vector V(s) of the original pencil sA - E.
C     REMARK: This auxiliary routine is intended to be called only from
C             the routine COLRED.
C
C     ARGUMENTS IN
C
C        MA - INTEGER.
C            The number of rows of matrix A.
C            MA >= 1.
C        NA - INTEGER.
C            The number of columns of matrix A.
C            NA >= MA.
C        I - INTEGER.
C            The actual number of columns in Y, which is also the index
C            of the diagonal block in A corresponding to the null vector
C            Y(s).
C            I >= 1.
C        J - INTEGER.
C            The row index in A which corresponds to the null vector
C            Y(s).
C            J >= 1.
C        A - DOUBLE PRECISION array of DIMENSION (LDA,NA).
C            The leading (MA + 1) by NA part of this array must contain
C            the transformed matrix A' in the upper part and the
C            Householder transformation vectors in the lower part.
C        LDA - INTEGER.
C            The leading dimension of array A as declared in the calling
C            program.
C            LDA >= MA + 1.
C        S - INTEGER ARRAY of DIMENSION at least (J-1).
C            The leading J - 1 elements of this array must contain the
C            indices of the pivots of the right invertible diagonal
C            submatrices, i.e., the pivot of A(m,m) is A(m,S(m)),
C            M=1,...,J-1. S(m) is also the index of the column in array
C            A in which the m-th non-trivial Householder transformation
C            vector is stored.
C        Y - DOUBLE PRECISION array of DIMENSION (LDY,I).
C            The leading NA by I part of this array must contain the
C            polynomial null vector Y(s) of sA' - E to be transformed,
C            where the t-th column (t = 1,...,I) must contain the
C            coefficient of s**(t-1).
C            Note: this array is overwritten.
C        LDY - INTEGER.
c            The leading dimension of array Y as declared in the calling
C            program.
C            LDY >= NA.
C
C     ARGUMENTS OUT
C
C        Y - DOUBLE PRECISION array of DIMENSION (LDY,I).
C            The leading NA by I part of this array contains the
C            transformed null vector V(s) = Q * Y(s), where Q  is the
C            product of the (J-1) Householder transformations Q(m).
C
C     WORKSPACE
C
C        RWORK - DOUBLE PRECISION array of DIMENSION (2*MA).
C
C     METHOD
C
C     Let
C         Q(m)  = ( I    0  )
C                 ( 0  P(m) )
C     be the elementary Householder transformation corresponding to the
C     pivot A(m,S(m)), augmented such that Q(m) is NA by NA, then
C     Q(1) Q(2) ... Q(J-1) Y ==> Y is computed.
C
C     CONTRIBUTOR
C
C        A.J. Geurts.
C
C     REVISIONS
C
C        1992, October 27.
C        1994, December 8.
C
      IMPLICIT NONE
C     .. Parameters ..
      DOUBLE PRECISION ZERO, ONE
      PARAMETER (ZERO = 0.0D0, ONE = 1.0D0)
C     .. Scalar Arguments ..
      INTEGER MA, NA, I, J, LDA, LDY
C     .. Array Arguments ..
      INTEGER S(*)
      DOUBLE PRECISION A(LDA,NA), Y(LDY,I), RWORK(2*NA)
C     .. Local Scalars ..
      INTEGER LEN, M, M1, MY
      DOUBLE PRECISION TAU
C     .. External Subroutines/Functions ..
      EXTERNAL DCOPY, DGEMV, DGER, DLAVEC
C
C     .. Executable Statements ..
C
C     Compute the matrix P(m) Y by w = Y'u and next Y = Y - tau * uw'.
C     for m = j-1, ... , 1.
C
      DO 20 M = J - 1, 1, -1
         IF (A(M+1,S(M)) .NE. ZERO) THEN
            LEN = MA - M + 1
            MY = NA - MA + M
            M1 = MA + 1
            CALL DCOPY(LEN, A(M+1,S(M)), 1, RWORK, 1)
            CALL DLAVEC(LEN, ZERO, RWORK(M1), 1)
            TAU = RWORK(1)
            RWORK(1) = ONE
            CALL DGEMV('T', LEN, I, ONE, Y(MY,1), LDY, RWORK, 1,
     *                 ZERO, RWORK(M1), 1)
            CALL DGER(LEN, I, -TAU, RWORK, 1, RWORK(M1), 1, Y(MY,1),
     *                LDY)
         END IF
   20 CONTINUE
      RETURN
C *** Last line of COMPTV ***
      END

