C
      SUBROUTINE COMPTY(NP, NA, I, NYI, YI, MU, A, LDA, S, Y, LDY, SK,
     *                  RWORK, IERR)
C
C     PURPOSE
C
C     To compute a right null vector of the pencil sA' - E, where the
C     left part of A' is in staircase form. Actually, the computed
C     vector is the null vector of the corresponding left part of the
C     pencil.
C     REMARK: This auxiliary routine is intended to be called only from
C             the routine COLRED.
C
C     ARGUMENTS IN
C
C        NP - INTEGER.
C            The number of columns of the polynomial matrix.
C            NP >= 1.
C        NA - INTEGER.
C            The number of columns of the matrix A.
C            NA >= 1.
C        I - INTEGER.
C            The index of the current diagonal block A(i,i) of the
C            matrix A being transformed into staircase form.
C            I >= 1.
C        NYI - INTEGER.
C            The length of the righthandside vector YI.
C            1 <= NYI <= NP.
C        YI - DOUBLE PRECISION array of DIMENSION at least (NP).
C            The right null vector of A(i,i).
C        MU - INTEGER array of DIMENSION at least (MA).
C            MU(k), k = 1, ..., i must contain the row index of the left
C            upper element of A(k,k).
C        A - DOUBLE PRECISION array of DIMENSION (LDA,NA).
C            The leading MU(i) - 1 by MU(i) + NP - 1 part of this array
C            must contain the part of the matrix A' which is in
C            staircase form.
C        LDA - INTEGER.
C            The leading dimension of array A as declared in the calling
C            program.
C            LDA >= MU(i) - 1.
C        S - INTEGER array of DIMENSION at least (MA).
C            The leading MU(i) - 1 elements of this array must contain
C            the column indices of the pivots of the right invertible
C            diagonal matrices A(k,k), k = 1, ..., i-1.
C
C     ARGUMENTS OUT
C
C        Y - DOUBLE PRECISION array of DIMENSION (LDY,NA).
C            The leading NA by i part of this array contains the
C            computed polynomial right null vector Y(s) of sA' - E,
C            where the j-th column contains the coefficient of s**(j-1).
C            The last NA - MU(i) - NP + 1 components of Y(s) are zero.
C        LDY - INTEGER.
C            The leading dimension of array Y as declared in the calling
C            program.
C            LDY >= NA.
C
C     WORK SPACE
C
C        SK - INTEGER array of DIMENSION at least (NP).
C        RWORK - DOUBLE PRECISION array of DIMENSION at least (NP).
C
C     ERROR INDICATOR
C
C        IERR - INTEGER.
C           Unless the routine detects an error (see next section),
C           IERR contains 0 on exit.
C
C     WARNINGS AND ERRORS DETECTED BY THE ROUTINE
C
C     IERR = k : A(k,k) is not right invertible.
C
C     METHOD
C
C     Let the pencil sA' - E, partially transformed up to the i-th
C     block, be
C
C        (  sA(1,1) sA(1,2)-E(1) sA(1,3)      . . .  sA(1,i)  . . .    )
C        (          sA(2,2)      sA(2,3)-E(2) . . .  sA(2,i)  . . .    )
C        (                         .                   .               )
C        (                                .            .               )
C        (                                       .     .               )
C        (                                           sA(i,i)  . . .    )
C        (                                                    . . .    )
C        (                                                    . . .    )
C
C     where A(k,k), k = 1,..., i is right invertible, A(k,k+1) is square
C     and E(k) = I of appropriate size.
C     Let Y(i,i), the (constant) right null vector of A(i,i), be given.
C     Then the routine computes a right null vector of sA' - E of the
C     form
C
C        ( Y(1,1) .     Y(1,k) . . . Y(1,j) . . Y(1,i)   )
C        (           .
C        (              .
C        (              Y(k,k) . . . Y(k,j) . . Y(k,i)   )
C        (                 .
C        (                         .
C        (                                .     Y(i-1,i) )
C        (                                      Y(i,i)   )
C        (   0                                     0     )
C
C     by comparing coefficients of equal power of s in the formula
C             i            j                 k
C     A(k,k) SUM Y(k,j) * s  = Y(k+1,k+1) * s  +
C            j=k
C     i-1                j                    j    i                   i
C     SUM (Y(k+1,j+1) - SUM A(k,l) Y(l,j)) * s  - SUM A(k,l) Y(l,i) * s
C    j=k+1             l=k+1                     l=k+1
C
C     Y(k,j), j = k,..., i, is a vector of length MU(k) - MU(k-1).
C
C     CONTRIBUTOR
C
C        A.J. Geurts.
C
C     REVISIONS
C
C        1992, October 27.
C        1994, December 8.
C
      IMPLICIT NONE
C     .. Parameters ..
      DOUBLE PRECISION ZERO, ONE
      PARAMETER (ZERO = 0.0D0, ONE = 1.0D0)
C     .. Scalar Arguments ..
      INTEGER NP, NA, I, NYI, LDA, LDY, IERR
C     .. Array Arguments ..
      INTEGER MU(*), S(*), SK(*)
      DOUBLE PRECISION YI(*), A(LDA,*), Y(LDY,*), RWORK(*)
C     .. Local Scalars ..
      INTEGER J, K, M, INDEXK, MUK, MUK1, NUK, MUK1NP
C     .. External Subroutines ..
      EXTERNAL COMPYI, DCOPY, DGEMV, DLAVEC, DLASET
C
C     .. Executable Statements ..
C
      IERR = 0
C
C     Initialization of the polynomial null vector Y(s).
C
      CALL DLASET('G', NA, I, ZERO, ZERO, Y, LDY)
      K = I
      IF (I .EQ. 1) THEN
         CALL DCOPY(NYI, YI, 1, Y(1,1), 1)
      ELSE
         INDEXK = NP + MU(I-1)
         CALL DCOPY(NYI, YI, 1, Y(INDEXK,I), 1)
         DO 30 K = I - 1, 1, -1
            MUK = MU(K)
            INDEXK = NP + MUK
            NUK = MU(K+1) - MUK
            DO 20 J = K, I
C
C              Compute the righthandside for Y(k,j) and store the
C              result in RWORK.
C
               IF (J .LT. I) THEN
                  CALL DCOPY(NUK, Y(INDEXK,J+1), 1, RWORK, 1)
               ELSE
                  CALL DLAVEC(NUK, ZERO, RWORK, 1)
               END IF
               IF (J .GT. K) THEN
                  CALL DGEMV('N', NUK, MU(J)-MUK, -ONE, A(MUK,INDEXK),
     *                       LDA, Y(INDEXK,J), 1, ONE, RWORK, 1)
               END IF
C
C              Solve A(k,k) * Y(k,j) = RWORK for Y(k,j).
C
               IF (K .EQ. 1) THEN
                  CALL COMPYI(NUK, NP, A(1,1), LDA, S, RWORK, Y(1,J),
     *                       IERR)
               ELSE
                  MUK1 = MU(K-1)
                  MUK1NP = MUK1 + NP
                  DO 10 M = 1, NUK
                     SK(M) = S(MUK-1+M) - MUK1NP + 1
   10             CONTINUE
                  CALL COMPYI(NUK, MUK-MUK1, A(MUK,MUK1NP), LDA, SK,
     *                       RWORK, Y(MUK1NP,J), IERR)
               END IF
               IF (IERR .NE. 0) THEN
                  IERR = K
                  RETURN
               END IF
   20       CONTINUE
   30    CONTINUE
      END IF
      RETURN
C *** Last line of COMPTY ***
      END

