C
      SUBROUTINE COMPYI(M, N, A, LDA, S, V, Y, IERR)
C
C     PURPOSE
C
C     To compute a null vector yi of the right invertible diagonal
C     submatrix A(i,i), by solving an appropriate M by N system of
C     linear equations A y = v, where A is in staircase form.
C     REMARK: This auxiliary routine is intended to be called only from
C             the routine COLRED.
C
C     ARGUMENTS IN
C
C     M - INTEGER.
C         The number of rows of matrix A.
C         M >= 1.
C     N - INTEGER.
C         The number of columns of matrix A.
C         N >= M.
C     A - DOUBLE PRECISION array of DIMENSION (LDA,N).
C         The leading M by N part of this array must contain the
C         matrix A.
C     LDA - INTEGER.
C         The leading dimension of array A as declared by the calling
C         program.
C         LDA >= M.
C     S - INTEGER array of DIMENSION at least (M).
C         S(i), i = 1,...,M, must contain the column index of the corner
C         in the i-th row of A.
C     V - DOUBLE PRECISION array of DIMENSION at least (M).
C         The righthand-side of the system of linear equations.
C
C     ARGUMENTS OUT
C
C     Y - DOUBLE PRECISION array of DIMENSION at least (N).
C         The computed solution of the system of linear equation.
C
C     ERROR INDICATOR
C
C     IERR - INTEGER.
C         Unless the routine detects an error (see next section),
C         IERR contains 0 on exit.
C
C     WARNINGS AMD ERRORS DETECTED BY THE ROUTINE
C
C     IERR = 3 : The matrix A is not right invertible.
C
C     METHOD
C
C     Let A * P = ( B | Z ) where P is a permutation matrix such that B
C     is nonsingular upper triangular. Z contains the remaining columns
C     of A. Then the system B x = v is solved and y = P ( x | 0 )'.
C
C     CONTRIBUTOR
C
C        A.J. Geurts.
C
C     REVISIONS
C
C        1992, October 27.
C        1994, December 8.
C
      IMPLICIT NONE
C     .. Parameters ..
      DOUBLE PRECISION ZERO
      PARAMETER (ZERO = 0.0D0)
C     .. Scalar Arguments ..
      INTEGER M, N, LDA, IERR
C     .. Array Arguments ..
      INTEGER S(M)
      DOUBLE PRECISION A(LDA,N), V(M), Y(N)
C     .. Local Scalars ..
      INTEGER J, K, SI
      DOUBLE PRECISION SUM
      LOGICAL FAIL
C     .. External Subroutines ..
      EXTERNAL DLARDV, DLAVEC
      DOUBLE PRECISION DLARDV
C
C     .. Executable Statements ..
C
C     Check input parameters.
C
      IF (N .LT. M) THEN
         IERR = 3
         RETURN
      END IF
C
      IERR = 0
      CALL DLAVEC(N, ZERO, Y, 1)
      SI = S(M)
      Y(SI) = DLARDV(V(M), A(M,SI), FAIL)
      IF (FAIL) THEN
         IERR = 3
         RETURN
      END IF
      DO 20 K = M - 1, 1, -1
         SUM = V(K)
         DO 10 J = K + 1, M
            SI = S(J)
            SUM = SUM - A(K,SI) * Y(SI)
   10    CONTINUE
         SI = S(K)
         Y(SI) = DLARDV(SUM, A(K,SI), FAIL)
         IF (FAIL) THEN
            IERR = 3
            RETURN
         END IF
   20 CONTINUE
      RETURN
C *** Last line of COMPYI ***
      END

