C
      SUBROUTINE HHTRAN(MA, NA, L, K, A, LDA, Q, RWORK)
C
C     PURPOSE
C
C     To compute the Householder reflection Q which transforms a
C     subcolumn of A into the first unit vector and to apply Q left and
C     right to appropriate submatrices of A.
C     REMARK: This auxiliary routine is intended to be called only from
C             the routine COLRED.
C
C     ARGUMENTS IN
C
C        MA - INTEGER.
C            The number of rows of matrix A.
C            MA >= 1.
C        NA - INTEGER.
C            The number of columns of matrix A.
C            NA >= MA.
C        L - INTEGER.
C            The index of the column of A to be transformed.
C            L >= 1.
C        K - INTEGER.
C            The row index from which the column is to be transformed.
C            K >= 1.
C        A - DOUBLE PRECISION array of DIMENSION (LDA,NA).
C            The leading MA by NA part of this array must contain the
C            matrix A. The left lower MA - K + 1 by L - 1 block of the
C            matrix A is understood to be zero made by former
C            transformations.
C            Note: this array is overwritten.
C        LDA - INTEGER.
C            The leading dimension of array A as declared in the calling
C            program.
C            LDA >= MA.
C
C     ARGUMENTS OUT
C
C        A - DOUBLE PRECISION array of DIMENSION (LDA,NA).
C            The leading MA by NA part of this array contains the
C            transformed matrix Q A Q , where Q  is the Householder
C                                1   2         i
C            transformation appropriately augmented with an identity
C            matrix.
C        Q - DOUBLE PRECISION array of DIMENSION at least (MA-K+1).
C            The leading MA - K + 1 elements of this array contain the
C            constant tau and the vector v which define the Householder
C            reflection (see DLARGF, i.e., Q(1) contains the value tau
C            and Q(i), i=2,...MA-K+1 contain the vector v).
C
C     WORKSPACE
C
C        RWORK - DOUBLE PRECISION array of DIMENSION at least (MA-K+1).
C
C     METHOD
C
C     Let w be the subcolumn A(i,L), i=K,...,MA. Then the Householder
C     reflection P = I - tau * u * u', where u = (1, v')', such that
C     Pw = e1 (the first unit vector) is computed.
C     Let
C         Q  = ( I  0 )       Q  = ( I  0 )
C          1   ( 0  P )        2   ( 0  P )
C     be such that Q  is MA by MA and Q  is NA by NA.
C                   1                2
C     Then Q  A Q   is computed and stored in A.
C           1    2
C
C     CONTRIBUTOR
C
C        A.J. Geurts.
C
C     REVISIONS
C
C        1992, October 27.
C        1994, December 8.
C
      IMPLICIT NONE
C     .. Parameters ..
      DOUBLE PRECISION ZERO, ONE
      PARAMETER (ZERO = 0.0D0, ONE = 1.0D0)
C     .. Scalar Arguments ..
      INTEGER MA, NA, K, L, LDA
C     .. Array Arguments ..
      DOUBLE PRECISION A(LDA,NA), Q(MA-K+1), RWORK(MA-K+1)
C     .. Local Scalars ..
      INTEGER LEN, NMK, NL1
      DOUBLE PRECISION TAU
C     .. External Subroutines/Functions ..
      EXTERNAL DCOPY, DGEMV, DGER, DLARFG, DLAVEC
C
C     .. Executable Statements ..
C
      LEN = MA - K + 1
      IF (LEN .GT. 1) THEN
C
C        Generate the Householder transformation.
C
         CALL DCOPY(LEN, A(K,L), 1, Q, 1)
         CALL DLARFG(LEN, Q(1), Q(2), 1, TAU)
C
         IF (TAU .NE. ZERO) THEN
C
C           Compute matrix Q A by w = A'u and next A = A - tau * uw'.
C                           1
            Q(1) = ONE
            NL1 = NA - L + 1
            CALL DLAVEC(MA, ZERO, RWORK, 1)
            CALL DGEMV('T', LEN, NL1, ONE, A(K,L), LDA, Q, 1, ZERO,
     *                  RWORK, 1)
            CALL DGER(LEN, NL1, -TAU, Q, 1, RWORK, 1, A(K,L), LDA)
C
C           Compute matrix Q A Q  by w = Au and next A = A - tau * wu'.
C                           1   2
            NMK = NA - MA + K
            CALL DGEMV('N', MA, LEN, ONE, A(1,NMK), LDA, Q, 1, ZERO,
     *                  RWORK, 1)
            CALL DGER(MA, LEN, -TAU, RWORK, 1, Q, 1, A(1,NMK), LDA)
            Q(1) = TAU
         END IF
      ELSE
         Q(1) = ZERO
      END IF
      RETURN
C *** Last line of HHTRAN ***
      END

